// -*- mode: C++; c-file-style: "bsd"; tab-width: 4 -*-
// bootdata.c - Pilot routines for initial stroke database creation
// JStroke 1.x - Japanese Kanji handwriting recognition technology demo.
// Copyright (C) 1997  Robert E. Wells
// http://wellscs.com/pilot
// mailto:robert@wellscs.com
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program (gpl.html); if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Derived from prior work by Todd David Rudick on JavaDict and StrokeDic.
// Makes use of KANJIDIC data from Jim Breen of Monash University.
// Further credit details available at http://wellscs.com/pilot
// See readme.txt, changelo, and gpl.html for more information.
// ---------------------------------------------------------------------------

#define dsStrokeDicVersion "JStrokeDic 1.2\nOriginal strokedic Copyright (C) 1997 Todd David Rudick\nPilot adaptation Copyright (C) 1997 Robert E. Wells"

// ----- Kanji handwriting data ----------------------------------------------

#include "strokedata.h"

// ----- CreateStrokeDic -----------------------------------------------------

#ifdef FOR_STROKEDIC_CREATE
Err CreateStrokeDic(void) {
	char*         cpDicDbName = dcpDicDbName;
	UInt          index;
	VoidHand      RecHandle;
	Ptr           RecPointer;
	char          nullString = '\0';
	DmOpenRef     pDicDb;
	Err           error;
	LocalID       dbId;
	UInt          dbAttrs;
	UInt          iRecLen;
	CharPtr       cpVersion = dsStrokeDicVersion;
    CharPtr       cpRec;

	if ((error = DmCreateDatabase(diDicDbCardNo, cpDicDbName,
								  dcAppCreatorId, dcAppDbType,
								  false)))
		return error;
	if (!(dbId = DmFindDatabase(diDicDbCardNo, cpDicDbName)))
		return DmGetLastErr();

	// Set attributes so StrokeDic database will be backed up to PC...
	if ((error = DmDatabaseInfo(diDicDbCardNo, dbId,
								NULL, &dbAttrs, NULL, NULL, NULL, NULL,
								NULL, NULL, NULL, NULL, NULL)))
		return error;

	dbAttrs |= dmHdrAttrBackup;
	
	if ((error = DmSetDatabaseInfo(diDicDbCardNo, dbId,
								   NULL, &dbAttrs, NULL, NULL, NULL, NULL,
								   NULL, NULL, NULL, NULL, NULL)))
		return error;

	if (!(pDicDb = DmOpenDatabase(diDicDbCardNo, dbId, dmModeReadWrite)))
		return DmGetLastErr();

	// Make sure we have all our StrokeDic records created;
	// initialize them to have a single byte empty string, as
	// a good safe default.

	for (index = DmNumRecords(pDicDb); index <= diMaxStrokes; index++)
	{
		// Allocate for version string in 0 record, for empty string in rest.
		cpRec = (index == 0) ? cpVersion : &nullString;
		iRecLen = StrLen(cpRec)+1; // Count the null terminator...

		if (!(RecHandle = DmNewRecord(pDicDb, &index, iRecLen))) {
			error = DmGetLastErr();
			DmCloseDatabase(pDicDb);
			return error;
		}
		RecPointer = MemHandleLock(RecHandle); // Ignore possible error...

		(void) DmWrite(RecPointer, 0, cpRec, iRecLen);

		(void) MemPtrUnlock(RecPointer);
		(void) DmReleaseRecord(pDicDb, index, true);
	}

	DmCloseDatabase(pDicDb);
	return 0;
}
#endif /* FOR_STROKEDIC_CREATE */

// ----- LoadStrokeDic -------------------------------------------------------

#ifdef FOR_STROKEDIC_CREATE
Err LoadStrokeDic(DmOpenRef pDicDb) {
	UInt          iCurSize, iNewSize, iStrSize, iTotSize;
	CharPtr       cp, cpNewLine;
	char          cStrokeCnt;
	UInt          iStrokeCnt;
	VoidHand      RecHandle;
	Ptr           RecPointer;
	Err           error;
	char          nullString = '\0';

	for (cp = LoadStrokeDicPtr(); *cp; cp = cpNewLine+1)
	{
		// Use the first char to get stroke count, don't store it in the
		// database; we append the string to the record for that stroke
		// count, using the stroke count itself as the index.

		cStrokeCnt = *cp++;
		iStrokeCnt = cStrokeCnt - 'A' + 1;

		if (iStrokeCnt > diMaxStrokes) {
			ErrBox("Bad StrokeCnt char");
			return memErrNotEnoughSpace;
		}

		if (!(cpNewLine = StrChr(cp, '\n'))) {
			ErrBox("Missing newline");
			return memErrNotEnoughSpace;
		}

		iStrSize = cpNewLine - cp;

		if (!(RecHandle = DmGetRecord(pDicDb, iStrokeCnt))) {
			ErrBox("DmGetRecord err");
			return DmGetLastErr();
		}
		iCurSize = MemHandleSize(RecHandle);
		iNewSize = iCurSize + iStrSize;
		if ((error = DmReleaseRecord(pDicDb, iStrokeCnt, false))) {
			ErrBox("DmReleaseRecord err");
			return error;
		}
		if (!(RecHandle = DmResizeRecord(pDicDb, iStrokeCnt, iNewSize))) {
			ErrBox("DmResizeRecord err");
			return DmGetLastErr();
		}

		if (!(RecPointer = MemHandleLock(RecHandle))) {
			ErrBox("MemHandleLock err");
			return DmGetLastErr();
		}
		
		// Assumes that current size includes null char at end.
		iCurSize--; // Backup to index null byte at end of current...
		if ((error = DmWrite(RecPointer, iCurSize, cp, iStrSize))) {
			ErrBox("DmWrite err");
			return error;
		}
		
		// Write null char at the new end.
		if ((error = DmWrite(RecPointer, iCurSize+iStrSize, &nullString, 1))) {
			ErrBox("DmWrite null err");
			return error;
		}
		
		if ((error = MemPtrUnlock(RecPointer))) {
			ErrBox("MemPtrUnlock err");
			return error;
		}
		if ((error = DmReleaseRecord(pDicDb, iStrokeCnt, true))) {
			ErrBox("DmReleaseRecord err");
			return error;
		}
	}
	return 0;					// No error - all loaded!
}
#endif /* FOR_STROKEDIC_CREATE */
// ----- End of bootdata.c ---------------------------------------------------
